@@ -59,7 +59,9 @@
 #include "SignalManager.h"
 #include "SignalWidget.h"
 #include "UndoAction.h"
+#include "UndoAddLabelAction.h"
 #include "UndoDeleteAction.h"
+#include "UndoDeleteLabelAction.h"
 #include "UndoDeleteTrack.h"
 #include "UndoFileInfo.h"
 #include "UndoInsertAction.h"
@@ -78,7 +80,6 @@
 SignalManager::SignalManager(QWidget *parent)
     :QObject(),
     m_parent_widget(parent),
-    m_name(""),
     m_closed(true),
     m_empty(true),
     m_modified(false),
@@ -127,7 +128,6 @@ int SignalManager::loadFile(const KURL &url)
 {
     int res = 0;
     FileProgress *dialog = 0;
-    m_name = url.filename();
 
     // enter and stay in not modified state
     enableModifiedChange(true);
@@ -316,7 +316,7 @@ int SignalManager::save(const KURL &url, bool selection)
 	// maybe we now have a new mime type
 	m_file_info.set(INF_MIMETYPE, mimetype_name);
 
-	// check if we loose information and ask the user if this would
+	// check if we lose information and ask the user if this would
 	// be acceptable if so
 	QValueList<FileProperty> supported = encoder->supportedProperties();
 	QMap<FileProperty, QVariant> properties(m_file_info.properties());
@@ -330,13 +330,13 @@ int SignalManager::save(const KURL &url, bool selection)
 		qWarning("SignalManager::save(): unsupported property '%s'",
 		    m_file_info.name(it.key()).data());
 		all_supported = false;
-		lost_properties += m_file_info.name(it.key()) + "\n";
+		lost_properties += i18n(m_file_info.name(it.key())) + "\n";
 	    }
 	}
 	if (!all_supported) {
 	    // show a warning to the user and ask him if he wants to continue
 	    if (KMessageBox::warningContinueCancel(m_parent_widget,
-		i18n("Saving in this format will loose the following "
+		i18n("Saving in this format will lose the following "
 		     "additional file attribute(s):\n"
 		     "%1\n"
 		     "Do you still want to continue?").arg(
@@ -364,7 +364,6 @@ int SignalManager::save(const KURL &url, bool selection)
 	m_file_info.setRate(rate());
 	m_file_info.setBits(bits);
 	m_file_info.setTracks(tracks);
-	m_file_info.set(INF_FILENAME, filename);
 
 	if (!m_file_info.contains(INF_SOFTWARE) &&
 	    encoder->supportedProperties().contains(INF_SOFTWARE))
@@ -407,7 +406,37 @@ int SignalManager::save(const KURL &url, bool selection)
 	                 &src,   SLOT(cancel()));
 
 	// invoke the encoder...
-	if (!encoder->encode(m_parent_widget, src, dst, m_file_info)) {
+	bool encoded = false;
+
+	if (selection) {
+	    // we have to adjust all labels in the file info
+	    // use a copy, don't touch the original !
+	    FileInfo info = m_file_info;
+
+	    LabelList &labels = info.labels();
+	    LabelListIterator it(labels);
+	    while (Label *label = it.current()) {
+		unsigned int pos = label->pos();
+		++it;
+		if ((pos < ofs) || (pos >= ofs + len)) {
+		    // out of the selected area -> remove
+		    labels.remove(label);
+		} else {
+		    // move label left
+		    label->moveTo(pos - ofs);
+		}
+	    }
+
+	    // set the filename in the copy of the fileinfo, the original
+	    // file which is currently open keeps it's name
+	    info.set(INF_FILENAME, filename);
+	    encoded = encoder->encode(m_parent_widget, src, dst, info);
+	} else {
+	    // in case of a "save as" -> modify the current filename
+	    m_file_info.set(INF_FILENAME, filename);
+	    encoded = encoder->encode(m_parent_widget, src, dst, m_file_info);
+	}
+	if (!encoded) {
 	    KMessageBox::error(m_parent_widget,
 	        i18n("An error occurred while saving the file!"));
 	    res = -1;
@@ -491,9 +520,9 @@ void SignalManager::close()
     flushRedoBuffer();
 
     m_empty = true;
-    m_name = "";
     while (tracks()) deleteTrack(tracks()-1);
     m_signal.close();
+    m_file_info.clear();
 
     m_closed = true;
     m_selection.select(0,0);
@@ -845,6 +874,19 @@ void SignalManager::slotSamplesInserted(unsigned int track,
     m_last_length = m_signal.length();
 
     setModified(true);
+
+    // only adjust the labels once per operation
+    if (track == selectedTracks().at(0)) {
+	LabelListIterator it(labels());
+	while (Label *label = it.current()) {
+	    unsigned int pos = label->pos();
+	    if (pos >= offset) {
+		label->moveTo(pos + length);
+	    }
+	    ++it;
+	}
+    }
+
     emit sigSamplesInserted(track, offset, length);
     emitStatusInfo();
 }
@@ -859,6 +901,24 @@ void SignalManager::slotSamplesDeleted(unsigned int track,
     m_last_length = m_signal.length();
 
     setModified(true);
+
+    // only adjust the labels once per operation
+    if (track == selectedTracks().at(0)) {
+	LabelListIterator it(labels());
+	while (Label *label = it.current()) {
+	    unsigned int pos = label->pos();
+	    if (pos >= offset + length) {
+		// move label left
+		label->moveTo(pos - length);
+	    } else if ((pos >= offset) && (pos < offset+length)) {
+		// delete the label
+		deleteLabel(labelIndex(label), true);
+		continue;
+	    }
+	    ++it;
+	}
+    }
+
     emit sigSamplesDeleted(track, offset, length);
     emitStatusInfo();
 }
@@ -964,142 +1024,12 @@ void SignalManager::selectTrack(unsigned int track, bool select)
     }
 }
 
-//***************************************************************************
-/* ### -> should go to a "codec_ascii" plugin...
-
-int SignalManager::loadAscii()
-{
-    float value;
-    int cnt = 0;
-    float max = 0;
-    float amp;
-    int *sample = 0;
-
-    FILE *sigin = fopen(m_name.local8Bit(), "r");
-    if (!sigin) {
-	KMessageBox::error(0, i18n("File does not exist !"), i18n("Info"), 2);
-	return -ENOENT;
-    }
-
-    // scan in the first line for the sample rate
-    // ### to be done ###
-    rate = 44100;        //will be asked in requester
-
-    // scan for number of channels
-    m_channels = 1;
-
-    // loop over all samples in file to get maximum value
-    while (!feof(sigin)) {
-	if (fscanf (sigin, "%e\n", &value) == 1) {
-	    if ( value > max) max = value;
-	    if ( -value > max) max = -value;
-	    cnt++;
-	}
-    }
-    qDebug("SignalManager::loadAscii(): reading ascii file with %d samples",
-	  cnt);    // ###
-
-    // get the maximum and the scale
-    amp = (float)((1 << 23)-1) / max;
-    Signal *new_signal = new Signal(cnt, rate);
-    Q_ASSERT(new_signal);
-
-    if (new_signal) {
-	signal.append(new_signal);
-	new_signal->setBits(24);
-	sample = new_signal->getSample();
-    }
-
-    if (sample) {
-	fseek (sigin, 0, SEEK_SET);    //seek to beginning
-	cnt = 0;
-	while (!feof(sigin)) {
-	    if (fscanf(sigin, "%e\n", &value) == 1) {
-		sample[cnt++] = (int)(value * amp);
-	    }
-	}
-    }
-
-    fclose (sigin);
-
-    return 0;
-}
-### */
-
 //***************************************************************************
 void SignalManager::emitStatusInfo()
 {
     emit sigStatusInfo(length(), tracks(), rate(), bits());
 }
 
-/* ### -> should go to a "codec_ascii" plugin...
-
-int SignalManager::exportAscii(const char *name)
-{
-    Q_ASSERT(name);
-    if (!name) return ;
-
-    unsigned int length = getLength();
-    Q_ASSERT(length);
-    if (!length) return;
-
-    Q_ASSERT(m_channels);
-    if (!m_channels) return;
-
-    FILE *sigout = fopen(name, "w");
-    Q_ASSERT(sigout);
-    if (!sigout) return;
-
-    //prepare and show the progress dialog
-    char progress_title[256];
-    char str_channels[128];
-    if (m_channels == 1)
-	strncpy(str_channels, i18n("Mono"), sizeof(str_channels));
-    else if (m_channels == 2)
-	strncpy(str_channels, i18n("Stereo"), sizeof(str_channels));
-    else
-	snprintf(str_channels, sizeof(str_channels), "%d-channel", m_channels);
-    snprintf(progress_title, sizeof(progress_title),
-	i18n("Exporting %s ASCII file :"),
-	str_channels);
-
-    QString title = i18n(progress_title);
-    ProgressDialog *dialog = new ProgressDialog(100, title);
-    delete[] title;
-    if (dialog) dialog->show();
-
-    // loop for writing data
-    int *sample;
-    int percent_count = 0;
-    const double scale_y = (1 << 23)-1;
-    for (unsigned int pos = 0; pos < length ; pos++) {
-	// loop over all channels
-	for (unsigned int channel=0; channel < m_channels; channel++) {
-	    sample = signal.at(channel)->getSample();
-	    if (!sample) continue;
-
-	    if (channel != 0) fprintf(sigout, ",");
-	    fprintf(sigout, "%0.8e", (double)sample[pos]/scale_y);
-	}
-	fprintf(sigout,"\n");
-
-	// update the progress bar
-	percent_count--;
-	if (dialog && (percent_count <= 0)) {
-	    percent_count = length / 200;
-	    float percent = (float)pos;
-	    percent /= (float)length;
-	    percent *= 100.0;
-	    dialog->setProgress (percent);
-	}
-    }
-
-    if (dialog) delete dialog;
-    fclose(sigout);
-    return -1;
-}
-### */
-
 //***************************************************************************
 PlaybackController &SignalManager::playbackController()
 {
@@ -1601,5 +1531,109 @@ void SignalManager::setFileInfo(FileInfo &new_info, bool with_undo)
     emitUndoRedoInfo();
 }
 
+//***************************************************************************
+Label *SignalManager::findLabel(unsigned int pos) const
+{
+    QPtrListIterator<Label> it(labels());
+    Label *label;
+    while ((label = it.current())) {
+	if (label->pos() == pos) return label; // found it
+	++it;
+    }
+    return 0; // nothing found
+}
+
+//***************************************************************************
+int SignalManager::labelIndex(const Label *label) const
+{
+    int index = 0;
+    QPtrListIterator<Label> it(labels());
+    while (const Label *l = it.current()) {
+	if (l == label) return index; // found it
+	index++;
+ 	++it;
+    }
+    return -1; // nothing found*/
+}
+
+//***************************************************************************
+Label *SignalManager::labelAtIndex(int index)
+{
+    return labels().at(index);
+}
+
+//***************************************************************************
+bool SignalManager::addLabel(unsigned int pos)
+{
+    // if there already is a label at the given position, do nothing
+    if (findLabel(pos)) return false;
+
+    // create a new label
+    Label *label = new Label(pos, "");
+    Q_ASSERT(label);
+    if (!label) return false;
+
+    // put the label into the list
+    labels().inSort(label);
+    emit sigLabelCountChanged();
+
+    // register the undo action
+    UndoTransactionGuard undo(*this, i18n("add label"));
+    UndoAddLabelAction *undo_add =
+	new UndoAddLabelAction(labelIndex(label));
+    if (!registerUndoAction(undo_add)) {
+	delete undo_add;
+	labels().remove(label);
+	emit sigLabelCountChanged();
+	return false;
+    }
+    return true;
+}
+
+//***************************************************************************
+Label *SignalManager::addLabel(unsigned int pos, const QString &name)
+{
+    // if there already is a label at the given position, do nothing
+    if (findLabel(pos)) return 0;
+
+    // create a new label
+    Label *label = new Label(pos, name);
+    Q_ASSERT(label);
+    if (!label) return 0;
+
+    // put the label into the list
+    labels().inSort(label);
+    emit sigLabelCountChanged();
+
+    return label;
+}
+
+//***************************************************************************
+void SignalManager::deleteLabel(int index, bool with_undo)
+{
+    Q_ASSERT(index >= 0);
+    Q_ASSERT(index < (int)labels().count());
+    if ((index < 0) || (index >= (int)labels().count())) return;
+
+    Label *label = labels().at(index);
+    Q_ASSERT(label);
+    if (!label) return;
+
+    // register the undo action
+    if (with_undo) {
+	UndoTransactionGuard undo(*this, i18n("delete label"));
+	UndoDeleteLabelAction *undo_del =
+	    new UndoDeleteLabelAction(*label);
+	if (!registerUndoAction(undo_del)) {
+	    delete undo_del;
+	    delete label;
+	    return;
+	}
+    }
+
+    labels().remove(label);
+    emit sigLabelCountChanged();
+}
+
 //***************************************************************************
 //***************************************************************************
