@@ -56,8 +56,6 @@
 #include "SignalManager.h"
 #include "MouseMark.h"
 #include "UndoTransactionGuard.h"
-#include "UndoAddLabelAction.h"
-#include "UndoDeleteLabelAction.h"
 #include "UndoModifyLabelAction.h"
 
 #ifdef DEBUG
@@ -127,7 +125,7 @@ SignalWidget::SignalWidget(QWidget *parent)
     m_zoom(0.0), m_playpointer(-1), m_last_playpointer(-1), m_redraw(false),
     m_inhibit_repaint(0), m_selection(0),
     m_signal_manager(this),
-    m_track_pixmaps(), m_labels(), m_pixmap(0),
+    m_track_pixmaps(), m_pixmap(0),
     m_mouse_mode(MouseNormal),
     m_mouse_down_x(0), m_repaint_timer(this),
     m_position_widget(this), m_position_widget_timer(this)
@@ -164,6 +162,10 @@ SignalWidget::SignalWidget(QWidget *parent)
 	unsigned int)),
 	this, SLOT(slotSamplesModified(unsigned int, unsigned int,
 	unsigned int)));
+    connect(sig, SIGNAL(sigLabelCountChanged()),
+	this, SLOT(hidePosition()));
+    connect(sig, SIGNAL(sigLabelCountChanged()),
+	this, SLOT(refreshMarkersLayer()));
 
     connect(&(sig->selection()), SIGNAL(changed(unsigned int, unsigned int)),
 	this, SLOT(slotSelectionChanged(unsigned int, unsigned int)));
@@ -232,7 +234,7 @@ SignalWidget::~SignalWidget()
     if (m_pixmap) delete m_pixmap;
     m_pixmap = 0;
 
-    m_labels.clear();
+    labels().clear();
 
     if (m_selection) delete m_selection;
     m_selection = 0;
@@ -243,12 +245,6 @@ SignalWidget::~SignalWidget()
     }
 }
 
-//***************************************************************************
-int SignalWidget::saveFile(const KURL &url, bool selection)
-{
-    return m_signal_manager.save(url, selection);
-}
-
 //***************************************************************************
 void SignalWidget::toggleTrackSelection(int track)
 {
@@ -296,8 +292,11 @@ bool SignalWidget::executeNavigationCommand(const QString &command)
     CASE_COMMAND("selectall")
 	selectRange(0, m_signal_manager.length());
     CASE_COMMAND("selectnext")
-	selectRange(m_signal_manager.selection().last()+1,
-	            m_signal_manager.selection().length());
+	if (m_signal_manager.selection().length())
+	    selectRange(m_signal_manager.selection().last()+1,
+	                m_signal_manager.selection().length());
+	else
+	    selectRange(m_signal_manager.length() - 1, 0);
     CASE_COMMAND("selectprev")
 	unsigned int ofs = m_signal_manager.selection().first();
 	unsigned int len = m_signal_manager.selection().length();
@@ -320,7 +319,7 @@ bool SignalWidget::executeNavigationCommand(const QString &command)
 	addLabel(pos);
     CASE_COMMAND("deletelabel")
 	int index = parser.toInt();
-	deleteLabel(index, true);
+	m_signal_manager.deleteLabel(index, true);
 //    CASE_COMMAND("chooselabel")
 //	Parser parser(command);
 //	markertype = globals.markertypes.at(parser.toInt());
@@ -334,8 +333,97 @@ bool SignalWidget::executeNavigationCommand(const QString &command)
 //	loadLabel();
 //    CASE_COMMAND("savelabel")
 //	saveLabel(command);
-//    CASE_COMMAND("expandtolabel")
-//	jumptoLabel();
+    CASE_COMMAND("expandtolabel")
+	unsigned int selection_left  = m_signal_manager.selection().first();
+	unsigned int selection_right = m_signal_manager.selection().last();
+	if (labels().isEmpty()) return false; // we need labels for this
+	LabelListIterator it(labels());
+	Label *label;
+	Label *label_left  = 0;
+	Label *label_right = 0;
+	// the last label <= selection start -> label_left
+	// the first label >= selection end  -> label_right
+	while ((label = it.current())) {
+	    ++it;
+	    unsigned int lp = label->pos();
+	    if (lp <= selection_left)
+		label_left = label;
+	    if ((lp >= selection_right) && (!label_right)) {
+		label_right = label;
+		break; // done
+	    }
+	}
+	// default left label = start of file
+	selection_left = (label_left) ?
+	    label_left->pos() : 0;
+	// default right label = end of file
+	selection_right = (label_right) ?
+	    label_right->pos() : m_signal_manager.length() - 1;
+	unsigned int length = selection_right - selection_left + 1;
+	selectRange(selection_left, length);
+
+    CASE_COMMAND("selectnextlabels")
+	unsigned int selection_left;
+	unsigned int selection_right = m_signal_manager.selection().last();
+	LabelListIterator it(labels());
+	Label *label;
+	Label *label_left  = 0;
+	Label *label_right = 0;
+	if (labels().isEmpty()) return false; // we need labels for this
+
+	// special case: nothing selected -> select up to the first label
+	if (selection_right == 0) {
+	    label_right = labels().first();
+	    selection_left = 0;
+	} else {
+	    // find the first label starting after the current selection
+	    for (; (label = it.current()); ++it) {
+		if (label->pos() >= selection_right) {
+		    // take it as selection start
+		    label_left  = label;
+		    // and it's next one as selection end (might be null)
+		    ++it;
+		    label_right = it.current();
+		    break;
+		}
+	    }
+	    // default selection start = last label
+	    if (!label_left) label_left = labels().last();
+	    if (!label_left) return false; // no labels at all !?
+	    selection_left = label_left->pos();
+	}
+	// default selection end = end of the file
+	selection_right = (label_right) ?
+	    label_right->pos() : m_signal_manager.length() - 1;
+	unsigned int length = selection_right - selection_left + 1;
+	selectRange(selection_left, length);
+
+    CASE_COMMAND("selectprevlabels")
+	unsigned int selection_left  = m_signal_manager.selection().first();
+	unsigned int selection_right = m_signal_manager.selection().last();
+	LabelListIterator it(labels());
+	Label *label;
+	Label *label_left  = 0;
+	Label *label_right = 0;
+	if (labels().isEmpty()) return false; // we need labels for this
+
+	// find the last label before the start of the selection
+	for (; (label = it.current()); ++it) {
+	    if (label->pos() > selection_left)
+		break; // done
+	    label_left  = label_right;
+	    label_right = label;
+	}
+	// default selection start = start of file
+	selection_left = (label_left) ?
+	    label_left->pos() : 0;
+	// default selection end = first label
+	if (!label_right) label_right = labels().first();
+	if (!label_right) return false; // no labels at all !?
+	selection_right = label_right->pos();
+	unsigned int length = selection_right - selection_left + 1;
+	selectRange(selection_left, length);
+
 //    CASE_COMMAND("markperiod")
 //	markPeriods(command);
 //    CASE_COMMAND("saveperiods")
@@ -462,8 +550,8 @@ void SignalWidget::close()
     m_signal_manager.playbackController().playbackStop();
     m_signal_manager.playbackController().reset();
 
-    // remove all layers
-    m_labels.clear();
+    // discard all labels
+    labels().clear();
 
     // clear the display
     m_track_pixmaps.setAutoDelete(true);
@@ -894,26 +982,35 @@ void SignalWidget::contextMenuEvent(QContextMenuEvent *e)
     Q_ASSERT(context_menu);
     if (!context_menu) return;
 
+    QPopupMenu *submenu_select = new QPopupMenu(this);
+    Q_ASSERT(submenu_select);
+    if (!submenu_select) return;
+
     QPopupMenu *submenu_label = new QPopupMenu(this);
     Q_ASSERT(submenu_label);
     if (!submenu_label) return;
 
     SignalManager *manager = &signalManager();
     KIconLoader icon_loader;
+    bool have_selection = (manager->selection().length() != 0);
+    bool have_labels = !labels().isEmpty();
 
     /* menu items common to all cases */
 
-    // undo/redo
+    // undo
     int id;
     id = context_menu->insertItem(
 	icon_loader.loadIcon("undo.png", KIcon::Toolbar),
-	i18n("&Undo"), this, SLOT(contextMenuEditUndo()), CTRL+Key_Z);
+	i18n("&Undo"), this, SLOT(contextMenuEditUndo()),
+	Qt::CTRL + Qt::Key_Z);
     if (!manager->canUndo())
 	context_menu->setItemEnabled(id, false);
 
+    // redo
     id = context_menu->insertItem(
 	icon_loader.loadIcon("redo.png", KIcon::Toolbar),
-	i18n("&Redo"), this, SLOT(contextMenuEditRedo()), CTRL+Key_Y);
+	i18n("&Redo"), this, SLOT(contextMenuEditRedo()),
+	Qt::CTRL + Qt::Key_Y);
     if (!manager->canRedo())
 	context_menu->setItemEnabled(id, false);
     context_menu->insertSeparator();
@@ -921,19 +1018,48 @@ void SignalWidget::contextMenuEvent(QContextMenuEvent *e)
     // cut/copy/paste
     id = context_menu->insertItem(
 	icon_loader.loadIcon("editcut.png", KIcon::Toolbar),
-	i18n("Cu&t"), this, SLOT(contextMenuEditCut()), CTRL+Key_X);
+	i18n("Cu&t"), this, SLOT(contextMenuEditCut()),
+	Qt::CTRL + Qt::Key_X);
     id = context_menu->insertItem(
 	icon_loader.loadIcon("editcopy.png", KIcon::Toolbar),
-	i18n("&Copy"), this, SLOT(contextMenuEditCopy()), CTRL+Key_C);
+	i18n("&Copy"), this, SLOT(contextMenuEditCopy()),
+	Qt::CTRL + Qt::Key_C);
     id = context_menu->insertItem(
 	icon_loader.loadIcon("editpaste.png", KIcon::Toolbar),
-	i18n("&Paste"), this, SLOT(contextMenuEditPaste()), CTRL+Key_V);
+	i18n("&Paste"), this, SLOT(contextMenuEditPaste()),
+	Qt::CTRL + Qt::Key_V);
     context_menu->insertSeparator();
 
     int mouse_x = mapFromGlobal(e->globalPos()).x();
     if (mouse_x < 0) mouse_x = 0;
     if (mouse_x >= width())  mouse_x = width()  - 1;
-    unsigned int len = manager->selection().length();
+
+    // Selection / &Save
+    int id_select_save = submenu_select->insertItem(
+	i18n("&Save..."), this, SLOT(contextMenuSaveSelection()));
+    submenu_select->setItemEnabled(id_select_save, have_selection);
+
+    // Selection / &Expand to labels
+    int id_select_expand_to_labels = submenu_select->insertItem(
+	i18n("&Expand to labels"), this,
+	SLOT(contextMenuSelectionExpandToLabels()), Qt::Key_E);
+    submenu_select->setItemEnabled(id_select_expand_to_labels, have_labels);
+
+    // Selection / to next labels
+    int id_select_next_labels = submenu_select->insertItem(
+	i18n("to next labels"), this,
+	SLOT(contextMenuSelectionNextLabels()),
+	Qt::SHIFT + Qt::CTRL + Qt::Key_N);
+    submenu_select->setItemEnabled(id_select_next_labels, have_labels);
+
+    // Selection / to previous labels
+    int id_select_prev_labels = submenu_select->insertItem(
+	i18n("to previous labels"), this,
+	SLOT(contextMenuSelectionPrevLabels()),
+	Qt::SHIFT + Qt::CTRL + Qt::Key_P);
+    submenu_select->setItemEnabled(id_select_prev_labels, have_labels);
+
+    context_menu->insertItem(i18n("&Selection"), submenu_select);
 
     // label handling
     int id_label_new = submenu_label->insertItem(
@@ -959,22 +1085,20 @@ void SignalWidget::contextMenuEvent(QContextMenuEvent *e)
 
 	pos = label->pos();
 	m_selection->set(pos, pos);
-    } else if (isSelectionBorder(mouse_x)) {
+    }
+
+    if (isSelectionBorder(mouse_x)) {
 	// context menu: do something with the selection border
 
 	// expand to next marker (right) ?
 	// expand to next marker (left) ?
-    } else if (isInSelection(mouse_x) && (len > 1)) {
-	// context menu: do something with the selection
-
-	id = context_menu->insertItem(
-	    i18n("&Save selection..."), this,
-	    SLOT(contextMenuSaveSelection()));
-
-	// convert selection to markers ?
     }
 
-    context_menu->insertItem("&Label", submenu_label);
+//     if (isInSelection(mouse_x) && have_selection) {
+// 	// context menu: do something with the selection
+//     }
+
+    context_menu->insertItem(i18n("&Label"), submenu_label);
 
     context_menu->exec(QCursor::pos());
     delete context_menu;
@@ -995,8 +1119,8 @@ void SignalWidget::contextMenuLabelDelete()
     Q_ASSERT(m_selection);
     if (!m_selection) return;
 
-    Label *label = findLabel(m_selection->left());
-    int index = labelIndex(label);
+    Label *label = m_signal_manager.findLabel(m_selection->left());
+    int index = m_signal_manager.labelIndex(label);
     forwardCommand(QString("deletelabel(%1)").arg(index));
 }
 
@@ -1006,7 +1130,7 @@ void SignalWidget::contextMenuLabelProperties()
     Q_ASSERT(m_selection);
     if (!m_selection) return;
 
-    Label *label = findLabel(m_selection->left());
+    Label *label = m_signal_manager.findLabel(m_selection->left());
     labelProperties(label);
 }
 
@@ -1342,7 +1466,8 @@ void SignalWidget::mouseMoveEvent(QMouseEvent *e)
 	    // yes, this code gives the nifty cursor change....
 	    if (label) {
 		setMouseMode(MouseAtSelectionBorder);
-		QString text = i18n("label #%1").arg(labelIndex(label));
+		QString text = i18n("label #%1").arg(
+		    m_signal_manager.labelIndex(label));
 		if (label->name().length())
 		    text += i18n(" (%1)").arg(label->name());
 		showPosition(text, label->pos(), samples2ms(label->pos()),
@@ -1475,7 +1600,7 @@ void SignalWidget::paintEvent(QPaintEvent *)
 	QPainter p;
 	p.begin(m_layer[LAYER_MARKERS]);
 
-	QPtrListIterator<Label> it(m_labels);
+	LabelListIterator it(labels());
 	Label *label;
 	while ((label = it.current())) {
 	    ++it;
@@ -1618,7 +1743,7 @@ unsigned int SignalWidget::pixels2samples(int pixels) const
 //***************************************************************************
 int SignalWidget::samples2pixels(int samples) const
 {
-    if (m_zoom==0.0) return 0;
+    if (m_zoom == 0.0) return 0;
     return (int)rint((double)samples / m_zoom);
 }
 
@@ -1636,7 +1761,7 @@ Label *SignalWidget::findLabelNearMouse(int x) const
     Label *nearest = 0;
     int dmin = pixels2samples(SELECTION_TOLERANCE) ^ 2;
 
-    QPtrListIterator<Label> it(m_labels);
+    LabelListIterator it(labels());
     Label *label;
     while ((label = it.current())) {
 	++it;
@@ -1660,70 +1785,26 @@ Label *SignalWidget::findLabelNearMouse(int x) const
 }
 
 //***************************************************************************
-Label *SignalWidget::findLabel(unsigned int pos) const
+void SignalWidget::addLabel(unsigned int pos)
 {
-    QPtrListIterator<Label> it(m_labels);
-    Label *label;
-    while ((label = it.current())) {
-	if (label->pos() == pos) return label; // found it
-	++it;
+    if (m_signal_manager.addLabel(pos)) {
+	// edit the label's properties
+	Label *label = m_signal_manager.findLabel(pos);
+	labelProperties(label);
+
+	refreshLayer(LAYER_MARKERS);
+	showPosition(0, 0, 0, QPoint(-1,-1));
     }
-    return 0; // nothing found
 }
 
-//***************************************************************************
-int SignalWidget::labelIndex(const Label *label) const
-{
-    int index = 0;
-    QPtrListIterator<Label> it(m_labels);
-    while (const Label *l = it.current()) {
-	if (l == label) return index; // found it
-	index++;
- 	++it;
-    }
-    return -1; // nothing found*/
-}
-
-//***************************************************************************
-Label *SignalWidget::labelAtIndex(int index)
-{
-    return m_labels.at(index);
-}
-
-//***************************************************************************
-// LabelType *findMarkerType (const char */*txt*/)
-// {
-//    int cnt = 0;
-//    LabelType *act;
-//
-//    for (act = globals.markertypes.first(); act; act = globals.markertypes.next()) {
-//	if (strcmp (act->name, txt) == 0) return act;
-//	cnt++;
-//    }
-//    qWarning("could not find Labeltype %s\n", txt);
-//     return 0;
-// }
-
 ////****************************************************************************
 //void SignalWidget::loadLabel()
 //{
-//    labels->clear();    //remove old Label...
+//    labels().clear();    //remove old Labels...
 //    appendLabel ();
 //}
 //
 ////****************************************************************************
-//void SignalWidget::appendLabel()
-//{
-//    QString name = KFileDialog::getOpenFileName (0, "*.label", this);
-//    //  if (!name.isNull())
-//    //    {
-//    //      char *comstr=catString ("loadbatch (",name,")");
-//    //      globals.port->putMessage (comstr);
-//    //    }
-//    refresh ();
-//}
-//
-////****************************************************************************
 //void SignalWidget::saveLabel (const char *typestring)
 //{
 //    QString name = KFileDialog::getSaveFileName (0, "*.label", this);
@@ -1755,7 +1836,7 @@ Label *SignalWidget::labelAtIndex(int index)
 //
 //	//ended writing of types, so go on with the labels...
 //
-//	for (tmp = labels->first(); tmp; tmp = labels->next())  //write out labels
+//	for (tmp = labels().first(); tmp; tmp = labels().next())  //write out labels
 //	{
 //	    fprintf (out, tmp->getCommand());
 //	    fprintf (out, "\n");
@@ -1775,7 +1856,7 @@ bool SignalWidget::labelProperties(Label *label)
     Q_ASSERT(dlg);
     if (!dlg) return false;
 
-    dlg->setLabelIndex(labelIndex(label));
+    dlg->setLabelIndex(m_signal_manager.labelIndex(label));
     dlg->setLabelPosition(label->pos(), m_signal_manager.length(),
 	m_signal_manager.rate());
     dlg->setLabelName(label->name());
@@ -1809,90 +1890,6 @@ bool SignalWidget::labelProperties(Label *label)
     return accepted;
 }
 
-//***************************************************************************
-void SignalWidget::addLabel(unsigned int pos)
-{
-    // if there already is a label at the given position, do nothing
-    if (findLabel(pos)) return;
-
-    // create a new label
-    Label *label = new Label(pos, "");
-    Q_ASSERT(label);
-    if (!label) {
-	KMessageBox::sorry(this, i18n("Out of memory"));
-	return;
-    }
-
-    // put the label into the list
-    m_labels.inSort(label);
-
-    // register the undo action
-    UndoTransactionGuard undo(m_signal_manager, i18n("add label"));
-    UndoAddLabelAction *undo_add =
-	new UndoAddLabelAction(*this, labelIndex(label));
-    if (!m_signal_manager.registerUndoAction(undo_add)) {
-	delete undo_add;
-	m_labels.remove(label);
-	return;
-    }
-
-    // edit the label's properties
-    labelProperties(label);
-
-    refreshLayer(LAYER_MARKERS);
-    showPosition(0, 0, 0, QPoint(-1,-1));
-}
-
-//***************************************************************************
-Label *SignalWidget::addLabel(unsigned int pos, const QString &name)
-{
-    // if there already is a label at the given position, do nothing
-    if (findLabel(pos)) return 0;
-
-    // create a new label
-    Label *label = new Label(pos, name);
-    Q_ASSERT(label);
-    if (!label) {
-	KMessageBox::sorry(this, i18n("Out of memory"));
-	return 0;
-    }
-
-    // put the label into the list
-    m_labels.inSort(label);
-    refreshLayer(LAYER_MARKERS);
-    showPosition(0, 0, 0, QPoint(-1,-1));
-
-    return label;
-}
-
-//***************************************************************************
-void SignalWidget::deleteLabel(int index, bool with_undo)
-{
-    Q_ASSERT(index >= 0);
-    Q_ASSERT(index < (int)m_labels.count());
-    if ((index < 0) || (index >= (int)m_labels.count())) return;
-
-    Label *label = m_labels.at(index);
-    Q_ASSERT(label);
-    if (!label) return;
-
-    // register the undo action
-    if (with_undo) {
-	UndoTransactionGuard undo(m_signal_manager, i18n("delete label"));
-	UndoDeleteLabelAction *undo_del =
-	    new UndoDeleteLabelAction(*this, *label);
-	if (!m_signal_manager.registerUndoAction(undo_del)) {
-	    delete undo_del;
-	    delete label;
-	    return;
-	}
-    }
-
-    m_labels.remove(label);
-    refreshLayer(LAYER_MARKERS);
-    showPosition(0, 0, 0, QPoint(-1,-1));
-}
-
 ////****************************************************************************
 //void SignalWidget::jumptoLabel ()
 // another fine function contributed by Gerhard Zintel
@@ -1904,7 +1901,7 @@ void SignalWidget::deleteLabel(int index, bool with_undo)
 //	int lmarker = signalmanage->getLMarker();
 //	int rmarker = signalmanage->getRMarker();
 //	bool RangeSelected = (rmarker - lmarker) > 0;
-//	if (labels) {
+//	if (!labels().isEmpty()) {
 //	    Label *tmp;
 //	    int position = 0;
 //	    for (tmp = labels->first(); tmp; tmp = labels->next())
@@ -1988,8 +1985,8 @@ void SignalWidget::deleteLabel(int index, bool with_undo)
 //	}
 //    }
 //}
-//
-////****************************************************************************
+
+//****************************************************************************
 //void SignalWidget::saveBlocks (int bit)
 //{
 //    if (signalmanage) {
@@ -2032,7 +2029,7 @@ void SignalWidget::deleteLabel(int index, bool with_undo)
 //	}
 //    }
 //}
-//
+
 ////****************************************************************************
 //void SignalWidget::markSignal (const char *str)
 //{
@@ -2310,23 +2307,11 @@ void SignalWidget::slotSamplesInserted(unsigned int track,
 {
 //     qDebug("SignalWidget(): slotSamplesInserted(%u, %u,%u)", track,
 // 	offset, length);
+    Q_UNUSED(track);
+    Q_UNUSED(offset);
+    Q_UNUSED(length);
 
-    // only adjust the labels once per operation
-    if (track != m_signal_manager.selectedTracks().at(0)) return;
-
-    unsigned int modified = 0;
-    QPtrListIterator<Label> it(m_labels);
-    while (Label *label = it.current()) {
-	unsigned int pos = label->pos();
-	if (pos >= offset) {
-	    label->moveTo(pos + length);
-	    modified++;
-	}
-	++it;
-    }
-
-    if (modified)
-	refreshLayer(LAYER_MARKERS);
+    refreshLayer(LAYER_MARKERS);
 }
 
 //***************************************************************************
@@ -2335,29 +2320,11 @@ void SignalWidget::slotSamplesDeleted(unsigned int track,
 {
 //    qDebug("SignalWidget(): slotSamplesDeleted(%u, %u...%u)", track,
 // 	offset, offset+length-1);
+    Q_UNUSED(track);
+    Q_UNUSED(offset);
+    Q_UNUSED(length);
 
-    // only adjust the labels once per operation
-    if (track != m_signal_manager.selectedTracks().at(0)) return;
-
-    unsigned int modified = 0;
-    QPtrListIterator<Label> it(m_labels);
-    while (Label *label = it.current()) {
-	unsigned int pos = label->pos();
-	if (pos >= offset + length) {
-	    // move label left
-	    label->moveTo(pos - length);
-	    modified++;
-	} else if ((pos >= offset) && (pos < offset+length)) {
-	    // delete the label
-	    deleteLabel(labelIndex(label), true);
-	    modified++;
-	    continue;
-	}
-	++it;
-    }
-
-    if (modified)
-	refreshLayer(LAYER_MARKERS);
+    refreshLayer(LAYER_MARKERS);
 }
 
 //***************************************************************************
@@ -2522,4 +2489,3 @@ void SignalWidget::dragMoveEvent(QDragMoveEvent* event)
 
 //***************************************************************************
 //***************************************************************************
-/* end of src/SignalWidget.cpp */
