@@ -131,7 +131,7 @@ unsigned int TopWidget::ZoomListPrivate::ms(int index)
 //***************************************************************************
 //***************************************************************************
 TopWidget::TopWidget(KwaveApp &main_app)
-    :KMainWindow(), m_app(main_app), m_url()
+    :KMainWindow(), m_app(main_app)
 {
     int id=1000; // id of toolbar items
     KIconLoader icon_loader;
@@ -190,9 +190,6 @@ TopWidget::TopWidget(KwaveApp &main_app)
     FileLoader loader(menufile);
     Q_ASSERT(loader.buffer());
     if (loader.buffer()) parseCommands(loader.buffer());
-    updateMenu();
-
-    updateRecentFiles();
 
     m_main_widget = new MainWidget(this);
     Q_ASSERT(m_main_widget);
@@ -440,6 +437,8 @@ TopWidget::TopWidget(KwaveApp &main_app)
 	this, SLOT(setUndoRedoInfo(const QString&, const QString&)));
     connect(signal_manager, SIGNAL(sigModified(bool)),
             this, SLOT(modifiedChanged(bool)));
+    connect(signal_manager, SIGNAL(sigLabelCountChanged()),
+	    this, SLOT(updateMenu()));
 
     // set the MainWidget as the main view
     setCentralWidget(m_main_widget);
@@ -484,6 +483,8 @@ TopWidget::TopWidget(KwaveApp &main_app)
     setStatusInfo(0,0,0,0);
     setUndoRedoInfo(0,0);
     setSelectedTimeInfo(0,0,0);
+    updateMenu();
+    updateRecentFiles();
 
     // check if the aRts dispatcher is functional. if not, we better
     // should exit now, as most of the plugins would not work
@@ -756,10 +757,10 @@ int TopWidget::parseCommands(const QByteArray &buffer)
 //***************************************************************************
 int TopWidget::revert()
 {
-    Q_ASSERT(m_url.isValid());
-    if (!m_url.isValid()) return -EINVAL;
+    KURL url(signalName());
+    Q_ASSERT(url.isValid());
+    if (!url.isValid()) return -EINVAL;
 
-    KURL url(m_url);
     return loadFile(url);
 }
 
@@ -794,7 +795,6 @@ bool TopWidget::closeFile()
     Q_ASSERT(m_main_widget);
     if (m_main_widget) m_main_widget->closeSignal();
 
-    m_url = KURL();
     updateCaption();
     m_zoomselect->clearEdit();
     emit sigSignalNameChanged(signalName());
@@ -823,7 +823,6 @@ int TopWidget::loadFile(const KURL &url)
 
     if (!m_main_widget->loadFile(url)) {
 	// succeeded
-	m_url = url;
 	updateCaption();
     } else {
 	// load failed
@@ -863,7 +862,9 @@ int TopWidget::saveFile()
     if (!m_main_widget) return -EINVAL;
 
     if (signalName() != NEW_FILENAME) {
-	res = m_main_widget->saveFile(m_url, false);
+	KURL url;
+	url = signalName();
+	res = signalManager().save(url, false);
 
 	// if saving in current format is not possible (no encoder),
 	// then try to "save/as" instead...
@@ -883,86 +884,86 @@ int TopWidget::saveFileAs(bool selection)
     Q_ASSERT(m_main_widget);
     if (!m_main_widget) return -EINVAL;
 
+    KURL current_url;
+    current_url = signalName();
     KwaveFileDialog dlg(":<kwave_save_as>", CodecManager::encodingFilter(),
-        this, "Kwave save file", true, m_url.prettyURL(), "*.wav");
+        this, "Kwave save file", true, current_url.prettyURL(), "*.wav");
     // dlg.setKeepLocation(true);
     dlg.setOperationMode(KFileDialog::Saving);
     dlg.setCaption(i18n("Save As"));
     if (dlg.exec() != QDialog::Accepted) return -1;
 
     KURL url = dlg.selectedURL();
-    if (!url.isEmpty()) {
-	QString name = url.path();
-	QFileInfo path(name);
+    if (url.isEmpty()) return 0;
 
-	// add the correct extension if necessary
-	if (!path.extension(false).length()) {
-	    QString ext = dlg.selectedExtension();
-	    QStringList extensions = QStringList::split(" ", ext);
-	    ext = extensions.first();
-	    name += ext.mid(1);
-	    path = name;
-	    url.setPath(name);
-	}
+    QString name = url.path();
+    QFileInfo path(name);
 
-	// check if the file exists and ask before overwriting it
-	// if it is not the old filename
-	if ((name != signalName()) && (path.exists())) {
-	    if (KMessageBox::warningYesNo(this,
-	        i18n("The file '%1' already exists. Do you really "\
-	        "want to overwrite it?").arg(name)) != KMessageBox::Yes)
-	    {
-		return -1;
-	    }
-	}
-
-	m_url = url;
-
-	// maybe we now have a new mime type
-	QString previous_mimetype_name = signalManager().fileInfo().get(
-	    INF_MIMETYPE).toString();
-
-	QString new_mimetype_name;
-	new_mimetype_name = CodecManager::whatContains(m_url);
-
-	if (new_mimetype_name != previous_mimetype_name) {
-	    // saving to a different mime type
-	    // now we have to do as if the mime type and file name
-	    // has already been selected to satisfy the fileinfo
-	    // plugin
-	    qDebug("TopWidget::saveAs(%s) - [%s] (previous:'%s')",
-		m_url.prettyURL().data(), new_mimetype_name.data(),
-		previous_mimetype_name.data() );
-
-	    // set the new mimetype
-	    signalManager().fileInfo().set(INF_MIMETYPE,
-	        new_mimetype_name);
-	    // save the old filename and set the new one
-	    QString old_filename = signalManager().fileInfo().get(
-	        INF_FILENAME).toString();
-	    signalManager().fileInfo().set(INF_FILENAME,
-	        m_url.prettyURL());
-
-	    // now call the fileinfo plugin with the new filename and
-	    // mimetype
-	    Q_ASSERT(m_plugin_manager);
-	    res = (m_plugin_manager) ?
-	        m_plugin_manager->setupPlugin("fileinfo") : -1;
-
-	    // restore the mime type and the filename
-	    signalManager().fileInfo().set(INF_MIMETYPE,
-	        previous_mimetype_name);
-	    signalManager().fileInfo().set(INF_FILENAME,
-	        m_url.prettyURL());
-	}
-
-	if (!res) res = m_main_widget->saveFile(m_url, selection);
-
-	updateCaption();
-	m_app.addRecentFile(signalName());
-	updateMenu();
+    // add the correct extension if necessary
+    if (!path.extension(false).length()) {
+	QString ext = dlg.selectedExtension();
+	QStringList extensions = QStringList::split(" ", ext);
+	ext = extensions.first();
+	name += ext.mid(1);
+	path = name;
+	url.setPath(name);
     }
 
+    // check if the file exists and ask before overwriting it
+    // if it is not the old filename
+    if ((name != signalName()) && (path.exists())) {
+	if (KMessageBox::warningYesNo(this,
+	    i18n("The file '%1' already exists. Do you really "\
+	    "want to overwrite it?").arg(name)) != KMessageBox::Yes)
+	{
+	    return -1;
+	}
+    }
+
+    // maybe we now have a new mime type
+    QString previous_mimetype_name = signalManager().fileInfo().get(
+	INF_MIMETYPE).toString();
+
+    QString new_mimetype_name;
+    new_mimetype_name = CodecManager::whatContains(url);
+
+    if (new_mimetype_name != previous_mimetype_name) {
+	// saving to a different mime type
+	// now we have to do as if the mime type and file name
+	// has already been selected to satisfy the fileinfo
+	// plugin
+	qDebug("TopWidget::saveAs(%s) - [%s] (previous:'%s')",
+	    url.prettyURL().data(), new_mimetype_name.data(),
+	    previous_mimetype_name.data() );
+
+	// set the new mimetype
+	signalManager().fileInfo().set(INF_MIMETYPE,
+	    new_mimetype_name);
+	// save the old filename and set the new one
+	QString old_filename = signalManager().fileInfo().get(
+	    INF_FILENAME).toString();
+	signalManager().fileInfo().set(INF_FILENAME,
+	    url.prettyURL());
+
+	// now call the fileinfo plugin with the new filename and
+	// mimetype
+	Q_ASSERT(m_plugin_manager);
+	res = (m_plugin_manager) ?
+	    m_plugin_manager->setupPlugin("fileinfo") : -1;
+
+	// restore the mime type and the filename
+	signalManager().fileInfo().set(INF_MIMETYPE,
+	    previous_mimetype_name);
+	signalManager().fileInfo().set(INF_FILENAME,
+	    url.prettyURL());
+    }
+
+    if (!res) res = signalManager().save(url, selection);
+
+    updateCaption();
+    m_app.addRecentFile(signalName());
+    updateMenu();
+
     return res;
 }
 
@@ -972,8 +973,6 @@ int TopWidget::newSignal(unsigned int samples, double rate,
 {
     // abort if the user pressed cancel
     if (!closeFile()) return -1;
-
-    m_url = "file:"+NEW_FILENAME;
     emit sigSignalNameChanged(signalName());
 
     m_main_widget->newSignal(samples, rate, bits, tracks);
@@ -988,7 +987,16 @@ int TopWidget::newSignal(unsigned int samples, double rate,
 //***************************************************************************
 QString TopWidget::signalName()
 {
-    return (m_url.isValid()) ? m_url.path() : QString("");
+    // if a file is loaded -> path of the URL if it has one
+    KURL url;
+    url = signalManager().fileInfo().get(INF_FILENAME).toString();
+    if (url.isValid()) return url.path();
+
+    // we have something, but no name yet
+    if (!signalManager().isClosed()) return QString(NEW_FILENAME);
+
+    // otherwise: closed, nothing loaded
+    return "";
 }
 
 //***************************************************************************
@@ -1229,6 +1237,10 @@ void TopWidget::updateMenu()
     // enable/disable all items that depend on having a file
     bool have_file = (signalName().length() != 0);
     m_menu_manager->setItemEnabled("@NOT_CLOSED", have_file);
+
+    // enable/disable all items that depend on having a label
+    bool have_labels = (!signalManager().fileInfo().labels().isEmpty());
+    m_menu_manager->setItemEnabled("@LABELS", have_labels);
 }
 
 //***************************************************************************
